<!DOCTYPE html>
<html lang="en">

  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="author" content="{{ site.author }}">
    {%- if description %}
    <meta name="description" content="{{ description }}">
    {% endif %}
    <title>{% block title %}{% endblock %}</title>
  </head>

  <body>
    <header>
      <h1>A Blog</h1>
      <nav>
        <ul>
          <li><a href="/">Blog</a></li>
          <li><a href="/atom.xml">Atom Feed</a></li>
        </ul>
      </nav>
    </header>

    <main>
    {% block content %}
    {% endblock %}
    </main>
  </body>

</html>

